(* $Id: gtkn.ml,v 1.1 2010-11-08 09:19:46 deraugla Exp $ *)

(* gtk without objects *)

external gdk_label_set_text : Gtk.obj [>`label] -> string -> unit
  = "ml_gtk_label_set_text";
external gdk_set_back_pixmap : Gdk.window -> Gdk.pixmap -> int -> unit = 
  "ml_gdk_window_set_back_pixmap";

type t 'a = { g_obj : Gtk.obj 'a};

type widget 'a = [ > `widget ] as 'a;
type window_dr = [ = `drawable | `gdkwindow ];
type pixmap_dr = [ = `drawable | `gdkpixmap ];
type drawable 'a = [ > `drawable ] as 'a;

value window title = do {
  let w =
    GtkWindow.Window.create [Gobject.param GtkWindow.Window.P.title title]
  in
  {g_obj = w}
};

value window_move w x y = GtkWindow.Window.move w.g_obj x y;
value window_add_accel_group w = GtkWindow.Window.add_accel_group w.g_obj;

value widget_destroy w = GtkBase.Object.destroy w.g_obj;
value widget_hide w = GtkBase.Widget.hide w.g_obj;
value widget_realize w = GtkBase.Widget.realize w.g_obj;
value widget_show w = GtkBase.Widget.show w.g_obj;

value container_add item w = GtkBase.Container.add item.g_obj w.g_obj;
value pack_start obj w =
  GtkPack.Box.pack obj.g_obj w.g_obj ?{from = Some `START} ?{expand = None}
    ?{fill = None} ?{padding = None}
;
value pack_end obj w =
  GtkPack.Box.pack obj.g_obj w.g_obj ?{from = Some `\END} ?{expand = None}
    ?{fill = None} ?{padding = None}
;

value alignment (xalign, xscale) (yalign, yscale) width height = do {
  let pl =
    [Gobject.param GtkBin.Alignment.P.xalign xalign;
     Gobject.param GtkBin.Alignment.P.xscale xscale;
     Gobject.param GtkBin.Alignment.P.yalign yalign;
     Gobject.param GtkBin.Alignment.P.yscale yscale]
  in
  let pl =
    if width <> 0 then
      [Gobject.param GtkBase.Widget.P.width_request width :: pl]
    else pl
  in
  let pl =
    if height <> 0 then
      [Gobject.param GtkBase.Widget.P.height_request height :: pl]
    else pl
  in
  let w = GtkBin.Alignment.create pl in
  GtkBase.Widget.show w;
  {g_obj = w}
};

value box dir = do {
  let w = GtkPack.Box.create dir [] in
  GtkBase.Widget.show w;
  {g_obj = w}
};

value drawing_area width height = do {
  let w = GtkMisc.DrawingArea.create [] in
  if width <> 0 || height <> 0 then
    GtkMisc.DrawingArea.size w ~{width} ~{height}
  else ();
  GtkBase.Widget.show w;
  {g_obj = w}
};

value frame text = do {
  let w =
    GtkBin.Frame.create
      [Gobject.param GtkBin.Frame.P.label (Some text);
       Gobject.param GtkBin.Frame.P.label_xalign 1.0]
  in
  GtkBase.Widget.show w;
  {g_obj = w}
};

value label text xpad ypad = do {
  let w =
    GtkMisc.Label.create
      [Gobject.param GtkMisc.Label.P.label text;
       Gobject.param GtkMisc.Misc.P.xpad xpad;
       Gobject.param GtkMisc.Misc.P.ypad ypad]
  in
  GtkBase.Widget.show w;
  {g_obj = w}
};

value label_set_text w = gdk_label_set_text w.g_obj;

value menu () = {g_obj = GtkMenu.Menu.create []};

value menu_bar () = do {
  let w = GtkMenu.MenuBar.create [] in
  GtkBase.Widget.show w;
  {g_obj = w}
};

value menu_item label = do {
  let w = GtkMenu.MenuItem.create ~{label} () in
  GtkBase.Widget.show w;
  {g_obj = w}
};

value menu_check_item label active = do {
  let w = GtkMenu.CheckMenuItem.create ~{label} () in
  GtkMenu.CheckMenuItem.set w ~{active};
  GtkBase.Widget.show w;
  {g_obj = w}
};

value menu_radio_item group label active = do {
  let w = GtkMenu.RadioMenuItem.create ?{group} ~{label} () in
  GtkMenu.CheckMenuItem.set w ~{active};
  GtkBase.Widget.show w;
  {g_obj = w}
};

value menu_separator_item () = do {
  let w = GtkMenu.MenuItem.separator_create () in
  GtkBase.Widget.show w;
  {g_obj = w}
};

value menu_append item_menu w = do {
  GtkMenu.MenuShell.insert item_menu.g_obj w.g_obj ~{pos = -1}
};

value menu_set_submenu button menu = do {
  GtkMenu.MenuItem.set_submenu button.g_obj menu.g_obj
};

value notebook () = do {
  let w = GtkPack.Notebook.create [] in
  GtkBase.Widget.show w;
  {g_obj = w}
};

value notebook_append_page noteb tab_label child = do {
  GtkPack.Notebook.insert_page_menu noteb.g_obj child.g_obj
    ~{tab_label = Gpointer.may_box (Some tab_label.g_obj) ~{f x = x}}
    ~{menu_label = Gpointer.optboxed None} ?{pos = None}
};

value notebook_remove_page noteb child_n =
  GtkPack.Notebook.remove_page noteb.g_obj child_n
;

value progress_bar () = do {
  let w = GtkRange.ProgressBar.create [] in
  GtkBase.Widget.show w;
  {g_obj = w}
};

value group w = Some w.g_obj;

value sgn_destroy = 
  {GtkSignal.name = "destroy"; classe = `gtk;
   marshaller = GtkSignal.marshal_unit}
;

value sgn_activate = 
  {GtkSignal.name = "activate"; classe = `menuitem;
   marshaller = GtkSignal.marshal_unit}
;

value widget_add_accelerator item accel_group modi key = do {
  GtkBase.Widget.add_accelerator item.g_obj
    ~{sgn = sgn_activate} accel_group ~{key} ~{modi} ~{flags = [`VISIBLE]}
};

value widget_remove_accelerator item accel_group modi key = do {
  GtkBase.Widget.remove_accelerator item.g_obj accel_group ~{key} ~{modi}
};

value signal_connect item sgn callback =
  let _ : GtkSignal.id = GtkSignal.connect ~{sgn} ~{callback} item.g_obj in
  ()
;

value set w prop valu = Gobject.set prop w.g_obj valu;
value get w prop = Gobject.get prop w.g_obj;

value drawable w = GtkBase.Widget.window w.g_obj;
value create_gc w = Gdk.GC.create w.g_obj;

value set_back_pixmap w p = gdk_set_back_pixmap w.g_obj p.g_obj 0;

value widget_window w = {g_obj = GtkBase.Widget.window w.g_obj};

value pixmap width height =
  let (window, depth, colormap) =
    let colormap = GtkBase.Widget.get_default_colormap () in
    (None, Gdk.Visual.depth (Gdk.Color.get_visual colormap), colormap)
  in
  let p = Gdk.Pixmap.create ?{window} ~{width} ~{height} ~{depth} () in
  {g_obj = p}
;

value draw_rectangle dr gc x y width height =
  Gdk.Draw.rectangle dr.g_obj gc ~{x} ~{y} ~{width} ~{height} ()
;
value fill_rectangle dr gc x y width height =
  Gdk.Draw.rectangle dr.g_obj gc ~{x} ~{y} ~{width} ~{height} ~{filled = True}
    ()
;

value draw_arc dr gc x y width height start angle =
  Gdk.Draw.arc dr.g_obj gc ~{x} ~{y} ~{width} ~{height} ~{start} ~{angle} ()
;

value draw_pixmap dr gc pixmap xsrc ysrc xdest ydest width height =
  Gdk.Draw.pixmap dr.g_obj gc ~{xsrc} ~{ysrc} ~{xdest} ~{ydest} ~{width}
    ~{height} pixmap.g_obj
;

value draw_point dr gc x y = Gdk.Draw.point dr.g_obj gc ~{x} ~{y};

value window_clear w = Gdk.Window.clear w.g_obj;

value set_cursor w curs = Gdk.Window.set_cursor w.g_obj curs;

value add_events w el = GtkBase.Widget.add_events w.g_obj el;
